searchData={"items":[{"type":"module","title":"Es6Maps","doc":"Enables ES6-like shorthand usage of Elixir maps.","ref":"Es6Maps.html"},{"type":"module","title":"Why? - Es6Maps","doc":"When writing code that heavily utilizes structures and passes complex objects through multiple layers, it's common to frequently use map literals.\nThis often results in repetitive code patterns such as `ctx = %{variable: variable, user: user, ...}` or `%{variable: variable, user: user, ...} = ctx`.\n\nI believe that introducing a shorthand form of object creation to Elixir enhances the language's ergonomics and is a natural extension of its existing map literals syntax.\nThis feature will be immediately familiar to JavaScript and Rust developers, and similar shorthands are present in other languages such as Go.","ref":"Es6Maps.html#module-why"},{"type":"module","title":"Can I easily revert if I don't like it? - Es6Maps","doc":"Sure! Just add this to `.formatter.exs` and `mix format` your codebase:\n\n```elixir\nplugins: [Es6Maps.Formatter]\nes6_maps: [map_style: :vanilla]\n```","ref":"Es6Maps.html#module-can-i-easily-revert-if-i-don-t-like-it"},{"type":"module","title":"Is there any runtime overhead? - Es6Maps","doc":"No; the shorthand map keys compile down to exactly the same bytecode as the \"vanilla-style\" maps.","ref":"Es6Maps.html#module-is-there-any-runtime-overhead"},{"type":"module","title":"Installation - Es6Maps","doc":"The package can be installed by adding `es6_maps` to your list of dependencies and compilers in `mix.exs`:\n\n```elixir\n# mix.exs\n\ndef project do\n  [\n    compilers: [:es6_maps | Mix.compilers()],\n    deps: deps()\n  ]\nend\n\ndef deps do\n  [\n    {:es6_maps, \"~> 1.0.0\"}\n  ]\nend\n```","ref":"Es6Maps.html#module-installation"},{"type":"module","title":"Usage - Es6Maps","doc":"","ref":"Es6Maps.html#module-usage"},{"type":"module","title":"Creating maps - Es6Maps","doc":"```elixir\niex> {hello, foo, bar} = {\"world\", 1, 2}\niex> %{hello, foo, bar: bar}\n%{hello: \"world\", foo: 1, bar: 2}\n```","ref":"Es6Maps.html#module-creating-maps"},{"type":"module","title":"Destructuring maps - Es6Maps","doc":"```elixir\niex> %{hello, foo} = %{hello: \"world\", foo: 1, bar: 2}\niex> hello\n\"world\"\niex> foo\n1\n```","ref":"Es6Maps.html#module-destructuring-maps"},{"type":"module","title":"Updating maps - Es6Maps","doc":"```elixir\niex> map = %{hello: \"world\", foo: 1, bar: 2}\niex> foo = :baz\niex> %{map | foo, bar: :bong}\n%{hello: \"world\", foo: :baz, bar: :bong}\n```","ref":"Es6Maps.html#module-updating-maps"},{"type":"module","title":"Structs - Es6Maps","doc":"All of the above work for structs as well:\n\n```elixir\ndefmodule MyStruct do\n  defstruct [:hello, :foo, :bar]\nend\n\niex> {foo, bar} = {1, 2}\niex> %MyStruct{foo, bar, hello: \"world\"}\n%MyStruct{foo: 1, bar: 2, hello: \"world\"}\n\niex> struct = %MyStruct{foo: 1, bar: 2}\niex> hello = \"world\"\niex> %MyStruct{struct | hello}\n%MyStruct{foo: 1, bar: 2, hello: \"world\"}\n\niex> %MyStruct{hello} = %MyStruct{hello: \"world\", foo: 1}\niex> hello\n\"world\"\n```","ref":"Es6Maps.html#module-structs"},{"type":"module","title":"Converting existing code to use ES6-style maps - Es6Maps","doc":"`es6_maps` includes a formatting plugin that will convert your existing map and struct literals into the shorthand style.\nAdd the plugin to `.formatter.exs`, then call `mix format` to reformat your code:\n\n```elixir\n# .formatter.exs\n[\n  plugins: [Es6Maps.Formatter],\n  inputs: [\"{mix,.formatter}.exs\", \"{config,lib,test}/**/*.{ex,exs}\"]\n]\n```\n\nThe plugin manipulates the AST, not raw strings, so it's precise and will only change your code by:\n\n1. changing map keys into the shorthand form;\n2. reordering map keys so the shorthand form comes first;\n3. formatting the results like `mix format` would.","ref":"Es6Maps.html#module-converting-existing-code-to-use-es6-style-maps"},{"type":"module","title":"Formatting pragmas - Es6Maps","doc":"The plugin supports pragmas in the comments to control the formatting.\nThe pragma must be in the form `# es6_maps: [map_style: :es6]` and can be placed anywhere in the file.\nThe `map_style` option can be set to `:es6` to convert to shorthand form or `:vanilla` to revert to the vanilla-style maps.\nThe pragma takes effect only on the line following the comment.\n\nFor example in the code below, the first map will be formatted to the shorthand form, while the second map will be left as is:\n\n```elixir\n  %{foo, bar: 1} = var\n  # es6_maps: [map_style: :vanilla]\n  %{hello: hello, foo: foo, bar: 1} = var\n```\n\n`es6_maps: [map_style: :vanilla]` option in `.formatter.exs` can be combined with `# es6_maps: [map_style: :es6]` comment pragmas.","ref":"Es6Maps.html#module-formatting-pragmas"},{"type":"module","title":"How does it work? - Es6Maps","doc":"`es6_maps` replaces in runtime the Elixir compiler's `:elixir` module.\nThe module's `string_to_tokens/5` function is wrapped with a function that replaces map keys `%{k}` as if they were `%{k: k}`.\nAfter `es6_maps` runs as one of the Mix compilers, the Elixir compiler will use the replaced functions to compile the rest of the code.\n\n> #### Important {: .info}\n>\n> By the nature of this solution it's tightly coupled to the internal Elixir implementation.\n> The current version of `es6_maps` should work for Elixir 1.15, 1.16, 1.17, 1.18 and the upcoming 1.19 version, but may break in the future.\n>\n> That said, starting with version 1.0.0 `es6_maps` instruments a very stable API that's unlikely to change.","ref":"Es6Maps.html#module-how-does-it-work"},{"type":"function","title":"Es6Maps.load/0","doc":"Injects the ES6 maps support into the Elixir compiler.","ref":"Es6Maps.html#load/0"},{"type":"function","title":"Es6Maps.start/2","doc":"Callback implementation for `Application.start/2`.  Calls `load/0`.","ref":"Es6Maps.html#start/2"},{"type":"module","title":"Es6Maps.Formatter","doc":"Replaces all map keys with their shorthand form.\n\nAdd the plugin to `.formatter.exs`, then call `mix format` to reformat your code:\n\n```elixir\n# .formatter.exs\n[\nplugins: [Es6Maps.Formatter],\ninputs: [\"{mix,.formatter}.exs\", \"{config,lib,test}/**/*.{ex,exs}\"]\n]\n```\n\nThe plugin manipulates the AST, not raw strings, so it's precise and will only change your code by:\n\n1. changing map keys into the shorthand form;\n2. reordering map keys so the shorthand form comes first;\n3. formatting the results like `mix format` would.","ref":"Es6Maps.Formatter.html"},{"type":"module","title":"Reverting to the vanilla-style maps - Es6Maps.Formatter","doc":"The formatting plugin can also be used to revert all of the ES6-style map shorthand uses back to the \"vanilla\" style.\nSet the `es6_maps: [map_style: :vanilla]` option in `.formatter.exs`, then call `mix format` to reformat your code:\n\n```elixir\n# .formatter.exs\n[\nplugins: [Es6Maps.Formatter],\ninputs: [\"{mix,.formatter}.exs\", \"{config,lib,test}/**/*.{ex,exs}\"],\nes6_maps: [map_style: :vanilla]\n]\n```","ref":"Es6Maps.Formatter.html#module-reverting-to-the-vanilla-style-maps"},{"type":"module","title":"Formatting pragmas - Es6Maps.Formatter","doc":"The plugin supports pragmas in the comments to control the formatting.\nThe pragma must be in the form `# es6_maps: [map_style: :vanilla]` and can be placed anywhere in the file.\nThe `map_style` option can be set to `:es6` to convert to shorthand form or `:vanilla` to revert to the vanilla-style maps.\nThe pragma takes effect only on the line following the comment.\n\nFor example in the code below, the first map will be formatted to the shorthand form, while the second map will be left as is:\n\n```elixir\n  %{foo, bar: 1} = var\n  # es6_maps: [map_style: :vanilla]\n  %{hello: hello, foo: foo, bar: 1} = var\n```\n\n`es6_maps: [map_style: :vanilla]` option in `.formatter.exs` can be combined with `# es6_maps: [map_style: :es6]` comment pragmas.","ref":"Es6Maps.Formatter.html#module-formatting-pragmas"},{"type":"module","title":"Options - Es6Maps.Formatter","doc":"* `es6_maps`:\n    * `map_style` - `:es6` to convert to shorthand form, `:vanilla` to revert to the vanilla-style maps.\n  * all other options of mix format, such as `line_length`, are supported and passed down to formatting functions.","ref":"Es6Maps.Formatter.html#module-options"},{"type":"task","title":"mix compile.es6_maps","doc":"Injects the ES6 maps support into the Elixir compiler.\n\nThis module does nothing more than calling `Es6Maps.load/0`, causing the ES6 maps support\nto be injected into the Elixir compiler from that moment on.\n\nIt should be added as the first in the list of `:compilers` in `Mix.exs`:\n\n    def project do\n      [\n        compilers: [:es6_maps | Mix.compilers()],\n        deps: deps()\n      ]\n    end","ref":"Mix.Tasks.Compile.Es6Maps.html"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"},{"type":"extras","title":"NOTICE","doc":"LEGAL NOTICE INFORMATION\n------------------------\n\nAll the files in this distribution are copyright to the terms below.\n\nCopyright 2024 Konrad Zemek  \n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    https://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language","ref":"notice.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}